@@ -167,6 +167,22 @@ retry:
 	return (int)h.data;
 }
 
+static int recv_header(int fd, struct sm_header *h)
+{
+	ssize_t rv;
+
+retry:
+	rv = recv(fd, h, sizeof(struct sm_header), MSG_WAITALL);
+	if (rv == -1 && errno == EINTR)
+		goto retry;
+	if (rv < 0)
+		return -errno;
+	if (rv != sizeof(struct sm_header))
+		return -1;
+
+	return 0;
+}
+
 static int cmd_lockspace(int cmd, struct sanlk_lockspace *ls, uint32_t flags, uint32_t data)
 {
 	int rv, fd;
@@ -370,15 +386,29 @@ int sanlock_get_hosts(const char *ls_name, uint64_t host_id,
 	return rv;
 }
 
-int sanlock_set_config(const char *ls_name, uint32_t flags, uint32_t cmd, GNUC_UNUSED void *data)
+int sanlock_set_config(const char *ls_name, uint32_t flags, uint32_t cmd, void *config_data)
 {
 	struct sanlk_lockspace ls;
 	struct sm_header h;
+	uint32_t h_data2 = 0;
 	int rv, fd;
 
 	if (!ls_name)
 		return -EINVAL;
 
+	/*
+	 * For SANLK_CONFIG_IO_TIMEOUT:
+	 * data arg points to 4 byte io_timeout value
+	 */
+	if (cmd == SANLK_CONFIG_IO_TIMEOUT) {
+		if (!config_data)
+			return -EINVAL;
+		memcpy(&h_data2, config_data, 4);
+		/* leader.io_timeout is uint16_t */
+		if (!h_data2 || (h_data2 > 0x000000FF))
+			return -EINVAL;
+	}
+
 	memset(&ls, 0, sizeof(struct sanlk_lockspace));
 	strncpy(ls.name, ls_name, SANLK_NAME_LEN);
 
@@ -388,7 +418,7 @@ int sanlock_set_config(const char *ls_name, uint32_t flags, uint32_t cmd, GNUC_U
 
 	rv = send_header(fd, SM_CMD_SET_CONFIG, flags,
 			 sizeof(struct sanlk_lockspace),
-			 cmd, 0);
+			 cmd, h_data2);
 	if (rv < 0)
 		goto out;
 
@@ -441,6 +471,77 @@ int sanlock_align(struct sanlk_disk *disk)
 	return rv;
 }
 
+int sanlock_read_lockspace_host(struct sanlk_lockspace *ls, uint32_t flags, uint32_t *io_timeout,
+				struct sanlk_host *hs)
+{
+	struct sm_header h;
+	int rv, fd;
+
+	if (!ls || !ls->host_id_disk.path[0])
+		return -EINVAL;
+
+	rv = connect_socket(&fd);
+	if (rv < 0)
+		return rv;
+
+	rv = send_header(fd, SM_CMD_READ_LOCKSPACE_HOST, flags,
+			 sizeof(struct sanlk_lockspace),
+			 0, 0);
+	if (rv < 0)
+		goto out;
+
+	rv = send_data(fd, ls, sizeof(struct sanlk_lockspace), 0);
+	if (rv < 0) {
+		rv = -errno;
+		goto out;
+	}
+
+	memset(&h, 0, sizeof(h));
+
+	rv = recv_data(fd, &h, sizeof(h), MSG_WAITALL);
+	if (rv < 0) {
+		rv = -errno;
+		goto out;
+	}
+
+	if (rv != sizeof(h)) {
+		rv = -1;
+		goto out;
+	}
+
+	rv = (int)h.data;
+	if (rv < 0)
+		goto out;
+
+	rv = recv_data(fd, ls, sizeof(struct sanlk_lockspace), MSG_WAITALL);
+	if (rv < 0) {
+		rv = -errno;
+		goto out;
+	}
+
+	if (rv != sizeof(struct sanlk_lockspace)) {
+		rv = -1;
+		goto out;
+	}
+
+	rv = recv_data(fd, hs, sizeof(struct sanlk_host), MSG_WAITALL);
+	if (rv < 0) {
+		rv = -errno;
+		goto out;
+	}
+
+	if (rv != sizeof(struct sanlk_host)) {
+		rv = -1;
+		goto out;
+	}
+
+	*io_timeout = h.data2;
+	rv = (int)h.data;
+ out:
+	close(fd);
+	return rv;
+}
+
 int sanlock_read_lockspace(struct sanlk_lockspace *ls, uint32_t flags, uint32_t *io_timeout)
 {
 	struct sm_header h;
@@ -1115,6 +1216,100 @@ int sanlock_killpath(int sock, uint32_t flags, const char *path, char *args)
 	return rv;
 }
 
+int sanlock_acquire2(int sock, int pid, uint32_t flags,
+		     struct sanlk_resource *res,
+		     struct sanlk_options *opt_in,
+		     struct sanlk_host *owner_host,
+		     char **owner_name)
+{
+	struct sm_header h = { 0 };
+	struct sanlk_options opt = {0};
+	char namebuf[SANLK_NAME_LEN+1] = {0};
+	char *nameret;
+	int result, rv, fd, data2;
+	int datalen;
+
+	if (res->num_disks != 1)
+		return -EINVAL;
+
+	if (opt_in)
+		memcpy(&opt, opt_in, sizeof(struct sanlk_options));
+
+	datalen = sizeof(struct sanlk_resource) + sizeof(struct sanlk_disk) + sizeof(struct sanlk_options);
+
+	if (sock == -1) {
+		/* connect to daemon and ask it to acquire a lease for
+		   another registered pid */
+
+		data2 = pid;
+
+		rv = connect_socket(&fd);
+		if (rv < 0)
+			return rv;
+	} else {
+		/* use our own existing registered connection and ask daemon
+		   to acquire a lease for self */
+
+		data2 = -1;
+		fd = sock;
+	}
+
+	rv = send_header(fd, SM_CMD_ACQUIRE2, flags, datalen, 1, data2);
+	if (rv < 0)
+		goto out;
+
+	rv = send_data(fd, res, sizeof(struct sanlk_resource), 0);
+	if (rv < 0) {
+		rv = -1;
+		goto out;
+	}
+
+	rv = send_data(fd, res->disks, sizeof(struct sanlk_disk), 0);
+	if (rv < 0) {
+		rv = -1;
+		goto out;
+	}
+
+	rv = send_data(fd, &opt, sizeof(struct sanlk_options), 0);
+	if (rv < 0) {
+		rv = -1;
+		goto out;
+	}
+
+	rv = recv_header(fd, &h);
+	if (rv < 0)
+		goto out;
+
+	result = (int)h.data;
+
+	if (h.length == sizeof(struct sm_header))
+		goto no_owner;
+
+	rv = recv_data(fd, owner_host, sizeof(struct sanlk_host), MSG_WAITALL);
+	if (rv < 0)
+		goto out;
+
+	rv = recv_data(fd, &namebuf, SANLK_NAME_LEN, MSG_WAITALL);
+	if (rv < 0)
+		goto out;
+
+	if (owner_name && namebuf[0]) {
+		int len = strlen(namebuf);
+		if ((nameret = malloc(len+1))) {
+			memcpy(nameret, namebuf, len);
+			nameret[len] = '\0';
+			*owner_name = nameret;
+		}
+	}
+
+ no_owner:
+	rv = result;
+ out:
+	if (sock == -1)
+		close(fd);
+	return rv;
+}
+
 int sanlock_acquire(int sock, int pid, uint32_t flags, int res_count,
 		    struct sanlk_resource *res_args[],
 		    struct sanlk_options *opt_in)
@@ -1165,7 +1360,7 @@ int sanlock_acquire(int sock, int pid, uint32_t flags, int res_count,
 
 	rv = send_header(fd, SM_CMD_ACQUIRE, flags, datalen, res_count, data2);
 	if (rv < 0)
-		return rv;
+		goto out;
 
 	for (i = 0; i < res_count; i++) {
 		res = res_args[i];
@@ -1236,7 +1431,7 @@ int sanlock_inquire(int sock, int pid, uint32_t flags, int *res_count,
 
 	rv = send_header(fd, SM_CMD_INQUIRE, flags, 0, 0, data2);
 	if (rv < 0)
-		return rv;
+		goto out;
 
 	/* get result */
 
