@@ -154,7 +154,7 @@ int delta_read_lockspace(struct task *task,
 			 uint64_t host_id,
 			 struct sanlk_lockspace *ls,
 			 int io_timeout,
-			 int *io_timeout_ret)
+			 struct sanlk_host *hs)
 {
 	struct leader_record leader_end;
 	struct leader_record leader;
@@ -192,7 +192,6 @@ int delta_read_lockspace(struct task *task,
 	if (error == SANLK_OK) {
 		memcpy(ls->name, leader.space_name, SANLK_NAME_LEN);
 		ls->host_id = host_id;
-		*io_timeout_ret = leader.io_timeout;
 
 		align_size = leader_align_size_from_flag(leader.flags);
 		if (!align_size)
@@ -204,6 +203,13 @@ int delta_read_lockspace(struct task *task,
 
 		ls->flags |= sanlk_lsf_sector_size_to_flag(leader.sector_size);
 		ls->flags |= sanlk_lsf_align_size_to_flag(align_size);
+
+		if (hs) {
+			hs->host_id = leader.owner_id;
+			hs->generation = leader.owner_generation;
+			hs->timestamp = leader.timestamp;
+			hs->io_timeout = leader.io_timeout;
+		}
 	}
 
 	return error;
@@ -299,11 +305,16 @@ int delta_lease_leader_clobber(struct task *task, int io_timeout,
 			       struct leader_record *leader,
 			       const char *caller)
 {
-	struct leader_record leader_end;
+	struct leader_record leader_end = { 0 };
+	uint32_t checksum;
 	int rv;
 
 	leader_record_out(leader, &leader_end);
 
+	checksum = leader_checksum(&leader_end);
+
+	leader_end.checksum = cpu_to_le32(checksum);
+
 	rv = write_sector(disk, leader->sector_size, host_id - 1, (char *)&leader_end, sizeof(struct leader_record),
 			  task, io_timeout, caller);
 	if (rv < 0)
@@ -332,6 +343,7 @@ int delta_lease_acquire(struct task *task,
 			char *space_name,
 			char *our_host_name,
 			uint64_t host_id,
+			int nodelay,
 			struct leader_record *leader_ret)
 {
 	struct leader_record leader;
@@ -394,13 +406,16 @@ int delta_lease_acquire(struct task *task,
 			no_delay = 1;
 		if (is_same && our_host_name_matches_product_uuid)
 			no_delay = 1;
+		if (nodelay)
+			no_delay = 1;
 
-		log_space(sp, "delta_acquire %s owner, %s free, %s our_product_uuid, %s delay, other_io_timeout %d",
+		log_space(sp, "delta_acquire %s owner, %s free, %s our_product_uuid, %s delay, other_io_timeout %d, flag_nodelay %d",
 			  is_same ? "same" : "new",
 			  is_free ? "is" : "not",
 			  our_host_name_matches_product_uuid ? "is" : "not",
 			  no_delay ? "no" : "short",
-			  other_io_timeout);
+			  other_io_timeout,
+			  nodelay);
 
 		goto write_new;
 	}
@@ -752,7 +767,7 @@ int delta_lease_renew(struct task *task,
 
 	/* We can't always memcmp(&leader, leader_last) because previous writes
 	   may have timed out and we don't know if they were actually written
-	   or not.  We can definately verify that we're still the owner,
+	   or not.  We can definitely verify that we're still the owner,
 	   though, which is the main thing we need to know. */
 
 	if (leader.owner_id != leader_last->owner_id ||
@@ -896,6 +911,107 @@ int delta_lease_release(struct task *task,
 	return SANLK_OK;
 }
 
+int delta_lease_init_host(struct task *task,
+		     struct sanlk_lockspace *ls,
+		     char *our_host_name,
+		     uint64_t generation,
+		     uint64_t timestamp,
+		     int io_timeout,
+		     struct sync_disk *disk)
+{
+	struct leader_record leader_end;
+	struct leader_record leader;
+	char *iobuf, **p_iobuf;
+	int iobuf_len;
+	int sector_size = 0;
+	int align_size = 0;
+	int max_hosts = 0;
+	int write_io_timeout;
+	uint32_t checksum;
+	uint64_t new_ts;
+	uint64_t offset;
+	int rv;
+
+	if (!io_timeout)
+		io_timeout = com.io_timeout;
+
+	if (timestamp == 0)
+		new_ts = LEASE_FREE;
+	else if (timestamp == 1)
+		new_ts = monotime();
+	else
+		new_ts = timestamp;
+
+	rv = sizes_from_flags(ls->flags, &sector_size, &align_size, &max_hosts, "LSF");
+	if (rv)
+		return rv;
+
+	if (!sector_size) {
+		/* sector/align flags were not set, use historical defaults */
+		sector_size = disk->sector_size;
+		align_size = sector_size_to_align_size_old(sector_size);
+		max_hosts = DEFAULT_MAX_HOSTS;
+	}
+
+	iobuf_len = sector_size;
+
+	p_iobuf = &iobuf;
+
+	rv = posix_memalign((void *)p_iobuf, getpagesize(), iobuf_len);
+	if (rv)
+		return rv;
+
+	memset(iobuf, 0, iobuf_len);
+
+	memset(&leader, 0, sizeof(struct leader_record));
+	leader.magic = DELTA_DISK_MAGIC;
+	leader.version = DELTA_DISK_VERSION_MAJOR | DELTA_DISK_VERSION_MINOR;
+	leader.flags = leader_align_flag_from_size(align_size);
+	leader.sector_size = sector_size;
+	leader.owner_id = ls->host_id;
+	leader.owner_generation = generation;
+	leader.max_hosts = 1;
+	leader.timestamp = new_ts;
+	leader.io_timeout = (io_timeout & 0x00FF);
+	memcpy(leader.space_name, ls->name, NAME_ID_SIZE);
+	if (our_host_name)
+		memcpy(leader.resource_name, our_host_name, NAME_ID_SIZE);
+	leader.checksum = 0; /* set below */
+
+	leader_record_out(&leader, &leader_end);
+
+	/*
+	 * N.B. must compute checksum after the data has been byte swapped.
+	 */
+	checksum = leader_checksum(&leader_end);
+	leader.checksum = checksum;
+	leader_end.checksum = cpu_to_le32(checksum);
+
+	memcpy(iobuf, &leader_end, sizeof(struct leader_record));
+	
+	/*
+	 * The io_timeout arg is a part of the lockspace logic, and
+	 * determines how the lockspace times out.  The process of
+	 * initializing the lease on disk can to use a longer timeout
+	 * than the algorithm uses.
+	 */
+	if (com.write_init_io_timeout)
+		write_io_timeout = com.write_init_io_timeout;
+	else
+		write_io_timeout = io_timeout;
+
+	offset = disk->offset + (sector_size * (ls->host_id - 1));
+
+	rv = write_iobuf(disk->fd, offset, iobuf, iobuf_len, task, write_io_timeout, NULL);
+	if (rv < 0)
+		goto out;
+ out:
+	if (rv != SANLK_AIO_TIMEOUT)
+		free(iobuf);
+
+	return rv;
+}
+
 /* the host_id lease area begins disk->offset bytes from the start of
    block device disk->path */
 
@@ -950,7 +1066,7 @@ int delta_lease_init(struct task *task,
 		leader.sector_size = sector_size;
 		leader.max_hosts = 1;
 		leader.timestamp = LEASE_FREE;
-		leader.io_timeout = io_timeout;
+		leader.io_timeout = (io_timeout & 0x00FF);
 		memcpy(leader.space_name, ls->name, NAME_ID_SIZE);
 		leader.checksum = 0; /* set below */
 
