@@ -24,6 +24,7 @@ int delta_lease_acquire(struct task *task,
                         char *space_name,
                         char *our_host_name,
                         uint64_t host_id,
+			int nodelay,
                         struct leader_record *leader_ret);
 
 int delta_lease_renew(struct task *task,
@@ -51,6 +52,14 @@ int delta_lease_init(struct task *task,
 		     int io_timeout,
 		     struct sync_disk *disk);
 
+int delta_lease_init_host(struct task *task,
+                     struct sanlk_lockspace *ls,
+		     char *our_host_name,
+                     uint64_t generation,
+                     uint64_t timestamp,
+                     int io_timeout,
+                     struct sync_disk *disk);
+
 int delta_read_lockspace(struct task *task,
 			struct sync_disk *disk,
 			int sector_size_hint,
@@ -58,7 +67,7 @@ int delta_read_lockspace(struct task *task,
 			uint64_t host_id,
 			struct sanlk_lockspace *ls,
 			int io_timeout,
-			int *io_timeout_ret);
+			struct sanlk_host *hs);
 
 int delta_read_lockspace_sizes(struct task *task,
                          struct sync_disk *disk,
