@@ -293,6 +293,8 @@ static int do_delta_action(int action,
 			   struct task *task,
 			   int io_timeout,
 			   struct sanlk_lockspace *ls,
+			   uint64_t generation,
+			   uint64_t timestamp,
 			   char *our_host_name,
 			   struct leader_record *leader_in,
 			   struct leader_record *leader_ret)
@@ -350,6 +352,11 @@ static int do_delta_action(int action,
 		rv = delta_lease_init(task, ls, io_timeout, &sd);
 		break;
 
+	case ACT_DIRECT_INIT_HOST:
+
+		rv = delta_lease_init_host(task, ls, our_host_name, generation, timestamp, io_timeout, &sd);
+		break;
+
 	case ACT_ACQUIRE_ID:
 		if (!sector_size || !align_size) {
 			rv = direct_read_leader_sizes(task, &sd, &sector_size, &align_size);
@@ -364,6 +371,7 @@ static int do_delta_action(int action,
 					 ls->name,
 					 our_host_name,
 					 ls->host_id,
+					 0,
 					 &leader);
 		break;
 	case ACT_RENEW_ID:
@@ -459,17 +467,17 @@ static int do_delta_action(int action,
 int direct_acquire_id(struct task *task, int io_timeout, struct sanlk_lockspace *ls,
 		      char *our_host_name)
 {
-	return do_delta_action(ACT_ACQUIRE_ID, task, io_timeout, ls, our_host_name, NULL, NULL);
+	return do_delta_action(ACT_ACQUIRE_ID, task, io_timeout, ls, 0, 0, our_host_name, NULL, NULL);
 }
 
 int direct_release_id(struct task *task, int io_timeout, struct sanlk_lockspace *ls)
 {
-	return do_delta_action(ACT_RELEASE_ID, task, io_timeout, ls, NULL, NULL, NULL);
+	return do_delta_action(ACT_RELEASE_ID, task, io_timeout, ls, 0, 0, NULL, NULL, NULL);
 }
 
 int direct_renew_id(struct task *task, int io_timeout, struct sanlk_lockspace *ls)
 {
-	return do_delta_action(ACT_RENEW_ID, task, io_timeout, ls, NULL, NULL, NULL);
+	return do_delta_action(ACT_RENEW_ID, task, io_timeout, ls, 0, 0, NULL, NULL, NULL);
 }
 
 int direct_align(struct sync_disk *disk)
@@ -489,7 +497,17 @@ int direct_write_lockspace(struct task *task, struct sanlk_lockspace *ls,
 	if (!ls)
 		return -1;
 
-	return do_delta_action(ACT_DIRECT_INIT, task, io_timeout, ls, NULL, NULL, NULL);
+	return do_delta_action(ACT_DIRECT_INIT, task, io_timeout, ls, 0, 0, NULL, NULL, NULL);
+}
+
+int direct_write_lockspace_host(struct task *task, struct sanlk_lockspace *ls, char *our_host_name,
+			   	uint64_t generation, uint64_t timestamp, uint32_t io_timeout)
+{
+	if (!ls)
+		return -1;
+
+	return do_delta_action(ACT_DIRECT_INIT_HOST, task, io_timeout, ls,
+			       generation, timestamp, our_host_name, NULL, NULL);
 }
 
 int direct_write_resource(struct task *task, struct sanlk_resource *res,
@@ -519,7 +537,7 @@ int direct_read_leader(struct task *task,
 	int rv = -1;
 
 	if (ls && ls->host_id_disk.path[0])
-		rv = do_delta_action(ACT_READ_LEADER, task, io_timeout, ls, NULL, NULL, leader_ret);
+		rv = do_delta_action(ACT_READ_LEADER, task, io_timeout, ls, 0, 0, NULL, NULL, leader_ret);
 
 	else if (res)
 		rv = do_paxos_action(ACT_READ_LEADER, task, io_timeout, res,
@@ -538,7 +556,7 @@ int direct_write_leader(struct task *task,
 	int rv = -1;
 
 	if (ls && ls->host_id_disk.path[0]) {
-		rv = do_delta_action(ACT_WRITE_LEADER, task, io_timeout, ls, NULL, leader, NULL);
+		rv = do_delta_action(ACT_WRITE_LEADER, task, io_timeout, ls, 0, 0, NULL, leader, NULL);
 
 	} else if (res) {
 		rv = do_paxos_action(ACT_WRITE_LEADER, task, io_timeout, res,
@@ -587,7 +605,7 @@ int direct_dump(struct task *task, char *dump_path, int force_mode)
 	 * /path[:<offset>[:<size>]]
 	 *
 	 * If path contains a colon, the user would escape it with \\, e.g.
-	 * device named /dev/foo:32 using offset 0 and lenth 1M would be
+	 * device named /dev/foo:32 using offset 0 and length 1M would be
 	 * /dev/foo\\:32:0:1M
 	 */
 
