@@ -37,6 +37,9 @@ int direct_align(struct sync_disk *disk);
 int direct_write_lockspace(struct task *task, struct sanlk_lockspace *ls,
 			   uint32_t io_timeout);
 
+int direct_write_lockspace_host(struct task *task, struct sanlk_lockspace *ls, char *our_host_name,
+                                uint64_t generation, uint64_t timestamp, uint32_t io_timeout);
+
 int direct_write_resource(struct task *task, struct sanlk_resource *res,
 			  int num_hosts, int write_clear);
 
