@@ -737,6 +737,8 @@ static void save_renewal_history(struct space *sp, int delta_result,
 		hi->timestamp = last_success;
 		hi->read_ms = rd_ms;
 		hi->write_ms = wr_ms;
+		hi->next_timeouts = 0;
+		hi->next_errors = 0;
 
 		sp->renewal_history_prev = sp->renewal_history_next;
 		sp->renewal_history_next++;
@@ -861,13 +863,15 @@ static void *lockspace_thread(void *arg_in)
 	int align_size = 0;
 	int max_hosts = 0;
 	int log_renewal_level = -1;
-	int rv, delta_length, renewal_interval = 0;
+	int cur_lockspace_io_timeout;
+	int delta_length, renewal_interval = 0;
 	int id_renewal_seconds, id_renewal_fail_seconds;
 	int acquire_result, delta_result, read_result;
 	int rd_ms, wr_ms;
 	int opened = 0;
 	int stop = 0;
 	int wd_con;
+	int rv;
 
 	if (com.debug_renew)
 		log_renewal_level = LOG_DEBUG;
@@ -878,6 +882,7 @@ static void *lockspace_thread(void *arg_in)
 	setup_task_aio(&task, main_task.use_aio, HOSTID_AIO_CB_SIZE);
 	memcpy(task.name, sp->space_name, NAME_ID_SIZE);
 
+	cur_lockspace_io_timeout = sp->io_timeout;
 	id_renewal_seconds = calc_id_renewal_seconds(sp->io_timeout);
 	id_renewal_fail_seconds = calc_id_renewal_fail_seconds(sp->io_timeout);
 
@@ -973,7 +978,7 @@ static void *lockspace_thread(void *arg_in)
 
 	delta_result = delta_lease_acquire(&task, sp, &sp->host_id_disk,
 					   sp->space_name, our_host_name_global,
-					   sp->host_id, &leader);
+					   sp->host_id, sp->nodelay, &leader);
 	delta_length = monotime() - delta_begin;
 
 	if (delta_result == SANLK_OK)
@@ -1017,6 +1022,14 @@ static void *lockspace_thread(void *arg_in)
 	while (1) {
 		pthread_mutex_lock(&sp->mutex);
 		stop = sp->thread_stop;
+
+		/* sp->io_timeout can be updated while running by set_config */
+		if (cur_lockspace_io_timeout != sp->io_timeout) {
+			cur_lockspace_io_timeout = sp->io_timeout;
+			id_renewal_seconds = calc_id_renewal_seconds(sp->io_timeout);
+			id_renewal_fail_seconds = calc_id_renewal_fail_seconds(sp->io_timeout);
+		}
+
 		pthread_mutex_unlock(&sp->mutex);
 		if (stop)
 			break;
@@ -1152,7 +1165,7 @@ static void free_sp(struct space *sp)
 	free(sp);
 }
 
-int add_lockspace_start(struct sanlk_lockspace *ls, uint32_t io_timeout, struct space **sp_out)
+int add_lockspace_start(struct sanlk_lockspace *ls, uint32_t io_timeout, int nodelay, struct space **sp_out)
 {
 	struct space *sp, *sp2;
 	int listnum = 0;
@@ -1178,6 +1191,7 @@ int add_lockspace_start(struct sanlk_lockspace *ls, uint32_t io_timeout, struct
 	sp->host_id_disk.fd = -1;
 	sp->host_id = ls->host_id;
 	sp->io_timeout = io_timeout;
+	sp->nodelay = nodelay;
 	sp->set_bitmap_seconds = calc_set_bitmap_seconds(io_timeout);
 	pthread_mutex_init(&sp->mutex, NULL);
 
@@ -1561,7 +1575,7 @@ int get_lockspaces(char *buf, int *len, int *count, int maxlen)
  *
  * * Hosts that are failing and don't recover would be the same as prev.
  *
- * * For hosts thet are failing but recover, we'd return:
+ * * For hosts that are failing but recover, we'd return:
  *   UNKNOWN then FAIL then LIVE
  *
  *
@@ -1597,7 +1611,7 @@ int get_lockspaces(char *buf, int *len, int *count, int maxlen)
  * After 80 seconds, we'd return FAIL.  After 140 seconds we'd return DEAD.
  */
 
-static uint32_t get_host_flag(struct space *sp, struct host_status *hs)
+uint32_t get_host_flag(struct space *sp, struct host_status *hs)
 {
 	uint32_t other_io_timeout;
 	int other_host_fail_seconds, other_host_dead_seconds;
@@ -1647,6 +1661,32 @@ static uint32_t get_host_flag(struct space *sp, struct host_status *hs)
 	}
 }
 
+int get_host_local(struct sanlk_lockspace *ls, struct sanlk_host *host)
+{
+	struct space *sp;
+	struct host_status *hs;
+	int rv = 0;
+
+	pthread_mutex_lock(&spaces_mutex);
+	sp = _search_space(ls->name, NULL, 0, &spaces, NULL, NULL, NULL);
+	if (!sp) {
+		rv = -ENOENT;
+		goto out;
+	}
+
+	hs = &sp->host_status[sp->host_id - 1];
+
+	host->host_id = sp->host_id;
+	host->generation = sp->host_generation;
+	host->timestamp = hs->timestamp;
+	host->io_timeout = hs->io_timeout;
+	host->flags = get_host_flag(sp, hs);
+ out:
+	pthread_mutex_unlock(&spaces_mutex);
+
+	return rv;
+}
+
 int get_hosts(struct sanlk_lockspace *ls, char *buf, int *len, int *count, int maxlen)
 {
 	struct space *sp;
@@ -1712,7 +1752,7 @@ int get_hosts(struct sanlk_lockspace *ls, char *buf, int *len, int *count, int m
 	return rv;
 }
 
-int lockspace_set_config(struct sanlk_lockspace *ls, GNUC_UNUSED uint32_t flags, uint32_t cmd)
+int lockspace_set_config(struct sanlk_lockspace *ls, GNUC_UNUSED uint32_t flags, uint32_t cmd, uint32_t data)
 {
 	struct space *sp;
 	int rv;
@@ -1756,6 +1796,27 @@ int lockspace_set_config(struct sanlk_lockspace *ls, GNUC_UNUSED uint32_t flags,
 		sp->flags &= ~SP_USED_BY_ORPHANS;
 		rv = 0;
 		break;
+
+	case SANLK_CONFIG_IO_TIMEOUT:
+		if (sp->space_dead) {
+			rv = -ENOSPC;
+			break;
+		}
+		if (!data) {
+			log_space(sp, "set_config %.48s zero io timeout is invalid", ls->name);
+			rv = -EINVAL;
+			break;
+		}
+		if (sp->io_timeout == data) {
+			log_space(sp, "set_config %.48s io_timeout is already %u", ls->name, sp->io_timeout);
+			rv = 0;
+			break;
+		}
+		log_space(sp, "set_config %.48s update io_timeout from %u to %u", ls->name, sp->io_timeout, data);
+		sp->io_timeout = data;
+		rv = 0;
+		break;
+
 	default:
 		rv = -EINVAL;
 	}
@@ -1851,7 +1912,7 @@ static int _clean_event_fds(struct space *sp)
 
 /*
  * end_event/reg_event don't need to worry about sp being freed
- * beause the main daemon thread processes end_event/reg_event,
+ * because the main daemon thread processes end_event/reg_event,
  * and the main thread is also the only thread that will free
  * sp structs.
  */
