@@ -42,7 +42,7 @@ int check_our_lease(struct space *sp, int *check_all, char *check_buf);
 void check_other_leases(struct space *sp, char *buf);
 
 /* locks spaces_mutex */
-int add_lockspace_start(struct sanlk_lockspace *ls, uint32_t io_timeout, struct space **sp_out);
+int add_lockspace_start(struct sanlk_lockspace *ls, uint32_t io_timeout, int nodelay, struct space **sp_out);
 
 /* locks sp, locks spaces_mutex */
 int add_lockspace_wait(struct space *sp);
@@ -64,6 +64,7 @@ int get_lockspaces(char *buf, int *len, int *count, int maxlen);
 
 /* locks spaces_mutex */
 int get_hosts(struct sanlk_lockspace *ls, char *buf, int *len, int *count, int maxlen);
+int get_host_local(struct sanlk_lockspace *ls, struct sanlk_host *host);
 
 /* locks spaces_mutex, locks sp */
 int lockspace_set_event(struct sanlk_lockspace *ls, struct sanlk_host_event *he, uint32_t flags);
@@ -78,9 +79,11 @@ int lockspace_end_event(struct sanlk_lockspace *ls);
 int send_event_callbacks(uint32_t space_id, uint64_t from_host_id, uint64_t from_generation, struct sanlk_host_event *he);
 
 /* locks spaces_mutex, locks sp */
-int lockspace_set_config(struct sanlk_lockspace *ls, uint32_t flags, uint32_t cmd);
+int lockspace_set_config(struct sanlk_lockspace *ls, uint32_t flags, uint32_t cmd, uint32_t data);
 
 int lockspace_begin_rindex_op(char *space_name, int rindex_op, struct space_info *spi);
 int lockspace_clear_rindex_op(char *space_name);
 
+uint32_t get_host_flag(struct space *sp, struct host_status *hs);
+
 #endif
