@@ -736,7 +736,9 @@ static int read_mode_block(struct task *task, struct token *token,
 }
 
 static int clear_dead_shared(struct task *task, struct token *token,
-			     int num_hosts, int *live_count)
+			     int num_hosts, int *live_count,
+			     uint64_t *first_live_host_id,
+			     uint64_t *first_live_host_gen)
 {
 	struct mode_block mb;
 	uint64_t host_id;
@@ -782,6 +784,11 @@ static int clear_dead_shared(struct task *task, struct token *token,
 			log_token(token, "clear_dead_shared host_id %llu gen %llu alive",
 				  (unsigned long long)host_id, (unsigned long long)mb.generation);
 			live++;
+
+			if (live == 1 && first_live_host_id && first_live_host_gen) {
+				*first_live_host_id = host_id;
+				*first_live_host_gen = mb.generation;
+			}
 			continue;
 		}
 
@@ -1077,7 +1084,7 @@ static int release_disk(struct task *task, struct token *token,
  * happening from this point forward.  However, another host contending for the
  * lease at the same time we failed, could already have read our dblock values
  * from before we cleared them.  In the worst case, that host could commit our
- * dblock values as the new leader, and that new leader write could apppear on
+ * dblock values as the new leader, and that new leader write could appear on
  * disk up to host_dead_seconds later.  So it seems that technically we would
  * need to monitor the leader for up to host_dead_seconds after clearing our
  * dblock to check if we become the on-disk owner of the lease.  The chances
@@ -1185,7 +1192,7 @@ static int _release_token(struct task *task, struct token *token,
 			ret = rv;
 		}
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 
 		/* Even when acquire did not get far enough to get a copy of the
@@ -1200,7 +1207,7 @@ static int _release_token(struct task *task, struct token *token,
 		if (rv < 0)
 			ret = rv;
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 
 		/* want to see this result in sanlock.log but not worry people with error */
@@ -1216,7 +1223,7 @@ static int _release_token(struct task *task, struct token *token,
 			ret = rv;
 		}
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 
 		rv = release_disk(task, token, resrename, &r->leader);
@@ -1225,7 +1232,7 @@ static int _release_token(struct task *task, struct token *token,
 			ret = rv;
 		}
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 
 	} else if (r_flags & R_SHARED) {
@@ -1237,7 +1244,7 @@ static int _release_token(struct task *task, struct token *token,
 			ret = rv;
 		}
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 
 	} else {
@@ -1271,7 +1278,7 @@ static int _release_token(struct task *task, struct token *token,
 			ret = rv;
 		}
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 	}
 
@@ -1301,7 +1308,7 @@ static int _release_token(struct task *task, struct token *token,
 	pthread_mutex_lock(&resource_mutex);
 	r->flags |= R_THREAD_RELEASE;
 	pthread_mutex_unlock(&resource_mutex);
-	return SANLK_AIO_TIMEOUT;
+	return ret;
 }
 
 static int release_token_nodisk_opened(struct task *task, struct token *token)
@@ -1570,7 +1577,7 @@ static int convert_sh2ex_token(struct task *task, struct resource *r, struct tok
 		goto do_mb;
 	}
 
-	rv = clear_dead_shared(task, token, leader.num_hosts, &live_count);
+	rv = clear_dead_shared(task, token, leader.num_hosts, &live_count, NULL, NULL);
 	if (rv < 0) {
 		log_errot(token, "convert_sh2ex clear_dead error %d", rv);
 		/* Do on-disk release of owner. Keep token and SH mblock. */
@@ -1795,12 +1802,15 @@ int convert_token(struct task *task, struct sanlk_resource *res, struct token *c
 }
 
 int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
-		  char *killpath, char *killargs)
+		  char *killpath, char *killargs,
+		  struct sanlk_host *owner_host)
 {
 	struct leader_record leader;
 	struct paxos_dblock dblock;
 	struct resource *r;
 	uint64_t acquire_lver = 0;
+	uint64_t first_live_host_id = 0;
+	uint64_t first_live_host_gen = 0;
 	uint32_t new_num_hosts = 0;
 	int sh_retries = 0;
 	int live_count = 0;
@@ -2031,11 +2041,23 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 			return SANLK_ACQUIRE_SHRETRY;
 		}
 		if (com.quiet_fail)
-			log_token(token, "acquire_token error: %d held", rv);
+			log_token(token, "acquire_token error: %d held by %llu %llu", rv,
+				  (unsigned long long)leader.owner_id,
+				  (unsigned long long)leader.owner_generation);
 		else
-			log_errot(token, "acquire_token error: %d held", rv);
+			log_errot(token, "acquire_token error: %d held by %llu %llu", rv,
+				  (unsigned long long)leader.owner_id,
+				  (unsigned long long)leader.owner_generation);
 		/* We must not write zero dblock values from write_host_block()! */
 		release_token_nodisk_opened(task, token);
+
+		if (owner_host) {
+			owner_host->host_id = leader.owner_id;
+			owner_host->generation = leader.owner_generation;
+			owner_host->timestamp = leader.timestamp;
+			owner_host->io_timeout = leader.io_timeout;
+		}
+
 		return rv;
 	}
 
@@ -2115,7 +2137,8 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 	 * with SHARED mb is dead, we clear it, otherwise it's alive
 	 * and we count it in live_count.
 	 */
-	rv = clear_dead_shared(task, token, leader.num_hosts, &live_count);
+	rv = clear_dead_shared(task, token, leader.num_hosts, &live_count,
+			       &first_live_host_id, &first_live_host_gen);
 	if (rv < 0) {
 		log_errot(token, "acquire_token clear_dead_shared error %d", rv);
 		release_token_opened(task, token);
@@ -2133,6 +2156,14 @@ int acquire_token(struct task *task, struct token *token, uint32_t cmd_flags,
 			log_errot(token, "acquire_token live_count release error %d", rv);
 			return rv;
 		}
+
+		if (owner_host) {
+			owner_host->host_id = first_live_host_id;
+			owner_host->generation = first_live_host_gen;
+			owner_host->timestamp = 0;
+			owner_host->io_timeout = 0;
+		}
+
 		return -EAGAIN;
 	}
 
@@ -2465,7 +2496,7 @@ static void resource_thread_release(struct task *task, struct resource *r, struc
 		if (rv < 0)
 			log_errot(token, "release async erase all write_host_block %d", rv);
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 
 		/* Even when acquire did not get far enough to get a copy of the
@@ -2477,7 +2508,7 @@ static void resource_thread_release(struct task *task, struct resource *r, struc
 		else
 			rv = paxos_lease_release(task, token, NULL, &r->leader, &leader);
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 
 		/* want to see this result in sanlock.log but not worry people with error */
@@ -2489,14 +2520,14 @@ static void resource_thread_release(struct task *task, struct resource *r, struc
 		if (rv < 0)
 			log_errot(token, "release async undo shared write_host_block %d", rv);
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 
 		rv = release_disk(task, token, NULL, &r->leader);
 		if (rv < 0)
 			log_errot(token, "release async undo shared release leader %d", rv);
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 
 	} else if (r_flags & R_SHARED) {
@@ -2506,7 +2537,7 @@ static void resource_thread_release(struct task *task, struct resource *r, struc
 		if (rv < 0)
 			log_errot(token, "release async shared write_host_block %d", rv);
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 	} else {
 		/* normal release of ex lease */
@@ -2529,7 +2560,7 @@ static void resource_thread_release(struct task *task, struct resource *r, struc
 		if (rv < 0)
 			log_errot(token, "release async release leader %d", rv);
 
-		if (rv == SANLK_AIO_TIMEOUT)
+		if (rv == SANLK_AIO_TIMEOUT || rv == -EIO)
 			retry_async = 1;
 	}
 
