@@ -12,6 +12,7 @@
 
 /* add flags */
 #define SANLK_ADD_ASYNC		0x00000001
+#define SANLK_ADD_NODELAY	0x00000002 /* can be unsafe */
 
 /* rem flags */
 #define SANLK_REM_ASYNC		0x00000001
@@ -23,6 +24,9 @@
 /* write flags */
 #define SANLK_WRITE_CLEAR	0x00000001 /* subsequent read will return error */
 
+/* get_hosts flags */
+#define SANLK_GET_HOST_LOCAL	0x00000001
+
 /* host status returned in low byte of sanlk_host.flags by get */
 #define SANLK_HOST_UNKNOWN 0x00000001
 #define SANLK_HOST_FREE    0x00000002
@@ -147,6 +151,7 @@ int sanlock_get_hosts(const char *ls_name, uint64_t host_id,
 #define SANLK_CONFIG_UNUSED		0x00000002
 #define SANLK_CONFIG_USED_BY_ORPHANS	0x00000004
 #define SANLK_CONFIG_UNUSED_BY_ORPHANS	0x00000008
+#define SANLK_CONFIG_IO_TIMEOUT		0x00000010
 
 int sanlock_set_config(const char *ls_name, uint32_t flags, uint32_t cmd, void *data);
 
@@ -256,11 +261,18 @@ int sanlock_write_lockspace(struct sanlk_lockspace *ls, int max_hosts,
  *
  * on success, zero is returned and
  * io_timeout and the entire sanlk_lockspace struct are written to
+ *
+ * For sanlock_read_lockspace_host(), the host fields are returned
+ * except for flags.
  */
 
 int sanlock_read_lockspace(struct sanlk_lockspace *ls,
 			   uint32_t flags, uint32_t *io_timeout);
 
+int sanlock_read_lockspace_host(struct sanlk_lockspace *ls,
+			   uint32_t flags, uint32_t *io_timeout,
+			   struct sanlk_host *hs);
+
 /*
  * format a resource lease area on disk
  *
