@@ -217,6 +217,7 @@ struct space {
 	uint32_t renewal_read_extend_sec; /* defaults to io_timeout */
 	uint32_t rindex_op;
 	unsigned int set_max_sectors_kb;
+	int nodelay;
 	int sector_size;
 	int align_size;
 	int max_hosts;
@@ -377,6 +378,7 @@ struct command_line {
 	int use_watchdog;
 	int watchdog_fire_timeout;
 	int io_timeout;			/* DEFAULT_IO_TIMEOUT or sanlock.conf io_timeout */
+	int io_timeout_set;
 	int kill_grace_seconds;		/* -g */
 	int kill_grace_set;
 	int high_priority;		/* -h */
@@ -408,6 +410,7 @@ struct command_line {
 	uint64_t host_generation;		/* -g */
 	uint64_t he_event;			/* -e */
 	uint64_t he_data;			/* -d */
+	uint64_t timestamp;			/* -t */
 	int num_hosts;				/* -n */
 	int max_hosts;				/* -m */
 	int res_count;
@@ -478,6 +481,7 @@ enum {
 	ACT_LOOKUP,
 	ACT_UPDATE,
 	ACT_REBUILD,
+	ACT_DIRECT_INIT_HOST,
 };
 
 EXTERN int external_shutdown;
