@@ -60,6 +60,8 @@ enum {
 	SM_CMD_CREATE_RESOURCE   = 38,
 	SM_CMD_DELETE_RESOURCE   = 39,
 	SM_CMD_REBUILD_RINDEX    = 40,
+	SM_CMD_ACQUIRE2		 = 41,
+	SM_CMD_READ_LOCKSPACE_HOST = 42,
 };
 
 #define SM_CB_GET_EVENT 1
